package cn.com.qmth.examcloud.service.examwork.api;

import io.swagger.annotations.ApiOperation;

import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.service.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.service.examwork.entity.Exam;
import cn.com.qmth.examcloud.service.examwork.service.ExamService;

/**
 * 考试服务API
 * Created by songyue on 17/1/13.
 */
@RestController
@RequestMapping("${app.api.root}")
public class ExamApi {

    @Autowired
    ExamRepo examRepo;

    @Autowired
    ExamService examService;

    @ApiOperation(value="查询所有考试批次",notes = "分页带查询")
    @GetMapping("/exam/all/{curPage}/{pageSize}")
    public ResponseEntity getAllExam(@ModelAttribute Exam examCriteria, @PathVariable Integer curPage,@PathVariable Integer pageSize){
    	return new ResponseEntity(examService.getAllExam(examCriteria,new PageRequest(curPage - 1,pageSize)), HttpStatus.OK);
    }

    @ApiOperation(value="查询所有考试批次",notes = "不分页带查询")
    @GetMapping("/exam/all")
    public ResponseEntity getAllExam(@ModelAttribute Exam examCriteria){
        return new ResponseEntity(examService.getAllExam(examCriteria), HttpStatus.OK);
    }

    @ApiOperation(value="按ID查询考试批次",notes = "ID查询")
    @GetMapping("/exam/{id}")
    public ResponseEntity<Exam> getExamById(@PathVariable Long id){
        return new ResponseEntity<Exam>(examService.getExamById(id),HttpStatus.OK);
    }

    @ApiOperation(value="新增考试批次",notes = "新增")
    @PostMapping("/exam")
    public ResponseEntity addExam(@RequestBody Exam exam){
        exam.setCreateTime(new Date());
        return new ResponseEntity(examService.saveExam(exam),HttpStatus.OK);
    }

    @ApiOperation(value="更新考试批次",notes = "更新")
    @PutMapping("/exam")
    public ResponseEntity updateExam(@RequestBody Exam exam){
        return new ResponseEntity(examService.saveExam(exam),HttpStatus.OK);
    }

    @ApiOperation(value="按ID删除考试批次",notes = "删除")
    @DeleteMapping("/exam/{id}")
    public ResponseEntity deleteExam(@PathVariable Long id){
    	examService.deleteExam(id);
        return new ResponseEntity(HttpStatus.OK);
    }
}
