package cn.com.qmth.examcloud.service.examwork.service;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.startsWith;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.service.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.service.examwork.entity.Exam;

/**
 * 考试批次服务类
 * Created by songyue on 17/1/13.
 */
@Service
public class ExamService {
    @Autowired
    ExamRepo examRepo;

    /**
     * 获取所有考试批次(分页)
     * @param examCriteria
     * @param pageable
     * @return
     */
    public Page<Exam> getAllExam(Exam examCriteria, Pageable pageable){
        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                .withMatcher("name",startsWith());
        Example<Exam> examExample = Example.of(examCriteria, exampleMatcher);
        return examRepo.findAll(examExample,pageable);
    }

    /**
     * 获取所有考试批次
     * @param examCriteria
     * @return 
     * @return
     */
    public List<Exam> getAllExam(Exam examCriteria){
    	ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                .withMatcher("name",startsWith());
        Example<Exam> examExample = Example.of(examCriteria,exampleMatcher);
        return examRepo.findAll(examExample);
    }

    /**
     * 获取所有考试批次
     * @return
     */
    public List<Exam> getAllExam(){
        return examRepo.findAll();
    }

    /**
     * 按ID获取考试批次
     * @param examId
     * @return
     */
    public Exam getExamById(Long examId){
        return examRepo.findOne(examId);
    }

    /**
     * 保存考试批次
     * @param exam
     * @return
     */
    public Exam saveExam(Exam exam){
        return examRepo.save(exam);
    }

    /**
     * 删除考试批次
     * @param examId
     * @return
     */
    public void deleteExam(Long examId){
        examRepo.delete(examId);
    }

    /**
     * 删除所有考试批次
     * @return
     */
    public void deleteAllExam(){
        examRepo.deleteAll();
    }
}
