package cn.com.qmth.examcloud.service.examwork.service;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.criteria.Predicate;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import cn.com.qmth.examcloud.common.dto.core.Course;
import cn.com.qmth.examcloud.common.dto.core.Org;
import cn.com.qmth.examcloud.common.dto.core.Student;
import cn.com.qmth.examcloud.common.dto.core.User;
import cn.com.qmth.examcloud.common.util.ErrorMsg;
import cn.com.qmth.examcloud.common.util.excel.ExcelError;
import cn.com.qmth.examcloud.common.util.excel.ExcelReader;
import cn.com.qmth.examcloud.common.util.excel.ExcelReaderHandle;
import cn.com.qmth.examcloud.service.examwork.assembler.ExamStudentAssembler;
import cn.com.qmth.examcloud.service.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.service.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.service.examwork.dto.ExamStudentDTO;
import cn.com.qmth.examcloud.service.examwork.entity.Exam;
import cn.com.qmth.examcloud.service.examwork.entity.ExamStudent;
import cn.com.qmth.examcloud.service.examwork.service.rpc.CourseService;
import cn.com.qmth.examcloud.service.examwork.service.rpc.OrgService;
import cn.com.qmth.examcloud.service.examwork.service.rpc.StudentService;

/**
 * 考试学生服务类
 * Created by songyue on 17/1/14.
 */
@Service
public class ExamStudentService {
    @Autowired
    ExamStudentRepo examStudentRepo;
    @Autowired
    ExamRepo examRepo;
    @Autowired
    ExamStudentAssembler examStudentAssembler;
    @Autowired
    StudentService studentService;
    @Autowired
    CourseService courseService;
    @Autowired
    OrgService orgService;
    
//    @Value("${app.em.photo.path}")
//    private String PHOTO_PATH ;
//    @Value("${app.em.facepp.key}")
//    private String FACEPP_KEY;
//    @Value("${app.em.facepp.secret}")
//    private String FACEPP_SECRET;
    
    /**
     * 获取所有考试学生(分页)
     * @param examCriteria
     * @param pageable
     * @return 
     * @return
     */
    public Page<ExamStudent> getAllExamStudent(ExamStudentDTO examCriteria, Pageable pageable){
//        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
//                .withMatcher("name",startsWith());
//        Example<ExamStudent> examExamStudentple = Example.of(examCriteria, exampleMatcher);
//        return examStudentRepo.findAll(examExamStudentple,pageable);
    	Specification<ExamStudent> specification = getSpecification(examCriteria);
        return examStudentRepo.findAll(specification,pageable);
    }

    /**
     * 获取所有考试学生
     * @param examCriteria
     * @return 
     * @return
     */
    public  List<ExamStudent> getAllExamStudent(ExamStudentDTO examCriteria){
    	Specification<ExamStudent> specification = getSpecification(examCriteria);
        return examStudentRepo.findAll(specification);
    }

    /**
     * 按ID获取考试学生
     * @param studentId
     * @return
     */
    public ExamStudent getExamStudentById(Long studentId){
        return examStudentRepo.findOne(studentId);
    }

    /**
     * 按考试批次获取学生
     * @param examId
     * @return 
     * @return
     */
    public Page<ExamStudent> getExamStudentByExamId(Long examId,Pageable pageable){
        return examStudentRepo.findByExamId(examId,pageable);
    }

    /**
     * 保存考试学生
     * @param exam
     * @return
     */
    public ExamStudent saveExamStudent(ExamStudent exam){
        return examStudentRepo.save(exam);
    }

    /**
     * 删除考试学生
     * @param examId
     * @return
     */
    public void deleteExamStudent(Long examId){
        examStudentRepo.delete(examId);
    }

    /**
     * 删除所有考试学生
     * @return
     */
    public void deleteAllExamStudent(){
        examStudentRepo.deleteAll();
    }
    
    /**
     * 导入考试学生
     * @return
     */
	public List<ExcelError> importExamStudent(Long examId, InputStream inputStream) {
		Exam exam = examRepo.findOne(examId);
		if(exam == null){
			new RuntimeException("请先创建考试");
		}
		List<ExamStudent> examStudents = new ArrayList<ExamStudent>();
		ExcelReader excelReader = new ExcelReader(ExamStudentDTO.class);
        List<ExcelError> excelErrors = excelReader.reader(inputStream, new ExcelReaderHandle() {
            @Override
            public ExcelError handle(Object obj) {
                ExamStudentDTO dto = (ExamStudentDTO) obj;
                dto.setRootOrgId(exam.getOrgId());
                dto.setExam(exam);
                dto.setFinished(false);
                dto.setGraduated(false);
                dto.setDegree(false);
                dto.setRepair(false);
                ExcelError error = importCheck(dto);
                if (error == null) {
                	examStudents.add(examStudentAssembler.toDomain(dto));
                }
                if (examStudents.size() % 1000 == 0) {
                    saveExamStudentAndUser(examStudents);
                }
                return error;
            }

        });
        saveExamStudentAndUser(examStudents);
        return excelErrors;
	}
	

	private void saveExamStudentAndUser(List<ExamStudent> examStudents) {
		for (ExamStudent examStudent : examStudents) {
			Student student = new Student();
			student.setIdentityNumber(examStudent.getIdentityNumber());
			student.setName(examStudent.getName());
			student.setStudentCode(examStudent.getStudentCode());
			User user = new User();
			user.setOrgId(examStudent.getOrgId());
			user.setRootOrgId(examStudent.getRootOrgId());
			student.setUser(user);
			student = studentService.addStudent(student);
			examStudent.setStudentId(student.getId());
			examStudentRepo.save(examStudent);
		}
		examStudents.removeAll(examStudents);
	}
	
	/**
	 * 考生导入验证
	 * @param stu
	 * @return
	 */
	public ExcelError importCheck(ExamStudentDTO dto){
		if(dto.getName()==null){
			return new ExcelError("姓名不能为空");
		}
		if(dto.getStudentCode()==null||dto.getIdentityNumber()==null){
			return new ExcelError("学号或身份证号不能为空");
		}
		if(dto.getCourseCode()==null){
			return new ExcelError("课程代码不能为空");
		}
		if(dto.getOrgCode()==null){
			return new ExcelError("学习中心代码不能为空");
		}
		Course course = courseService.findByOrgIdAndCode(dto.getRootOrgId(), dto.getCourseCode());
		if(course == null || !course.getEnable()){
			return new ExcelError("课程不存在/被禁用");
		}
		dto.setCourseLevel(course.getLevel().toString());
		dto.setCourseName(course.getName());
		Org org = orgService.findByParentIdAndCode(dto.getRootOrgId(), dto.getOrgCode());
		if(org == null || !org.getEnable()){
			return new ExcelError("学习中心不存在/禁用");
		}
		dto.setOrgId(org.getId());
		return null;
	}
	
	/**
	 * 照片检验
	 * @param examId
	 * @return
	 */
	public List<ErrorMsg> photoCheck(Long examId) {
		List<ErrorMsg> msgs = new ArrayList<ErrorMsg>();
		List<ExamStudent> examStudents = examStudentRepo.findByExamId(examId);
		for (ExamStudent examStudent : examStudents) {
			ErrorMsg errorMsg = null;
			if(!StringUtils.isEmpty(examStudent.getStudentCode())){
				
				errorMsg = new ErrorMsg(examStudent.getStudentCode());
			}else if(!StringUtils.isEmpty(examStudent.getIdentityNumber())){
				
				errorMsg =  new ErrorMsg(examStudent.getIdentityNumber());
			}
			if(errorMsg != null){
				msgs.add(errorMsg);
			}
		}
		return msgs;
	}
	
	/**
	 * 生成查询条件
	 * @param examCriteria
	 * @return
	 */
	private Specification<ExamStudent> getSpecification(ExamStudentDTO examCriteria) {
		Specification<ExamStudent> specification = (root, query, cb) -> {
		    List<Predicate> predicates = new ArrayList<>();
		    if(null!=examCriteria.getRootOrgId()){
		    	predicates.add(cb.equal(root.get("rootOrgId"),examCriteria.getRootOrgId()));
		    }
		    if(null!=examCriteria.getOrgId()){
		    	predicates.add(cb.equal(root.get("orgId"),examCriteria.getOrgId()));
		    }
		    if(null!=examCriteria.getExam()){
		    	predicates.add(cb.equal(root.get("exam").get("id"),examCriteria.getExam().getId()));
		    }
		    if(null!=examCriteria.getName()){
		    	predicates.add(cb.like(root.get("name"),"%"+examCriteria.getName()+"%"));
		    }
		    if(null!=examCriteria.getStudentCode()){
		    	predicates.add(cb.like(root.get("studentCode"),"%"+examCriteria.getStudentCode()+"%"));
		    }
		    if(null!=examCriteria.getFinished()){
		    	predicates.add(cb.equal(root.get("finished"),examCriteria.getFinished()));
		    }
		    if(null!=examCriteria.getStudentId()){
		    	predicates.add(cb.equal(root.get("studentId"),examCriteria.getStudentId()));
		    }
		    return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}
}
